import asyncio
import io
import json
import os
import pathlib
import time

import aiohttp
import cv2
import fastdeploy as fd
import numpy as np
from fastapi import APIRouter, HTTPException, UploadFile, WebSocket, WebSocketDisconnect
from fastapi.responses import StreamingResponse
from fastdeploy.serving.utils import cv2_to_base64

from image_identification.model import model_instance

router = APIRouter(prefix="/image")


async def request(url: str, data, headers):
    async with aiohttp.ClientSession() as session:
        async with session.post(url=url, data=data, headers=headers) as response:
            return await response.json()


# @router.post("/get_visuallized_image", tags=["analyze"])
# async def getVisualized_image(image: UploadFile):
#     contents = await image.read()
#     nparr = np.fromstring(contents, np.uint8)
#     im = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
#     url = "http://127.0.0.1:8000/fd/yolov5s"
#     headers = {"Content-Type": "application/json"}
#     data = {"data": {"image": cv2_to_base64(im)}, "parameters": {}}
#     try:
#         resp = await asyncio.gather(
#             *[request(url=url, data=json.dumps(data), headers=headers)]
#         )
#         r_json = json.loads(resp[0]["result"])
#         det_result = fd.vision.utils.json_to_detection(r_json)
#         vis_im = fd.vision.vis_detection(im, det_result, score_threshold=0.5)
#         _res, im_jpg = cv2.imencode(".jpg", vis_im)
#         return StreamingResponse(io.BytesIO(im_jpg.tobytes()), media_type="image/jpeg")
#     except:
#         raise HTTPException(status_code=500, detail="获取失败")


@router.post("/analyze", tags=["analyze"], summary="summary", description="description")
async def analyze(image: UploadFile):
    image_content = await image.read()
    nparr = np.fromstring(image_content, np.uint8)
    im = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
    result = model_instance.predict(im)
    vis_im = fd.vision.vis_detection(im, result, score_threshold=0.5)
    im_jpg = cv2.imencode(".jpg", vis_im)[1]
    return StreamingResponse(io.BytesIO(im_jpg.tobytes()), media_type="image/jpeg")


@router.websocket("/analyze_video")
async def analyze_video(websocket: WebSocket):
    rtmp_url = "rtmp://localhost/live/livestream"
    cap = cv2.VideoCapture(rtmp_url)
    frame_number = 0
    frame_rate = 30

    await websocket.accept()
    fourcc = cv2.VideoWriter.fourcc("M", "P", "4", "V")
    # os.mkdir("framers")
    print(cap.get(3))
    print(cap.get(4))
    os.chdir("framers")
    video = None
    try:
        while True:
            ret, frame = cap.read()
            if not ret:
                print("无法读取视频帧")
                break
            # 是否保存帧
            if True:
                # if frame_number % 4 == 0:
                # 处理每一帧的操作，这里只是保存为图像文件

                # cv2.imwrite(frame_filename, frame)
                result = model_instance.predict(frame)
                vis_im = fd.vision.vis_detection(frame, result, score_threshold=0.5)
                # cv2.imwrite(frame_filename, vis_im)
                if video is None:
                    video = cv2.VideoWriter(
                        "__output__.mp4",
                        fourcc,
                        30.0,
                        (vis_im.shape[1], vis_im.shape[0]),
                    )
                video.write(vis_im)
                print(vis_im.shape)
                await websocket.send_text(fd.vision.fd_result_to_json(result=result))
                # await websocket.send_text(frame_filename)
            else:
                print(frame.shape)
                video.write(frame)
            frame_number += 1
    except WebSocketDisconnect:
        print("close")
        await websocket.send_text("error")
        await websocket.close()
