import logging
import os
import time
import cv2
import uvicorn
from fastdeploy.serving.server import SimpleServer
import fastdeploy as fd
from .controllers import image
from .model import model_instance

logging.getLogger().setLevel(logging.INFO)

# Create server, setup REST API
app = SimpleServer()
app.register(
    task_name="fd/yolov5s",
    model_handler=fd.serving.handler.VisionModelHandler,
    predictor=model_instance,
)

app.include_router(image.router)


def run_app():
    uvicorn.run("image_identification.main:app", host="0.0.0.0", port=8000, reload=True)


async def analyze_video():
    rtmp_url = "rtmp://localhost/live/livestream"
    cap = cv2.VideoCapture(rtmp_url)
    frame_number = 0
    frame_rate = 30
    fps = int(cap.get(cv2.CAP_PROP_FPS))
    print(fps)
    fourcc = cv2.VideoWriter.fourcc("M", "P", "4", "V")
    # os.mkdir("framers")

    os.chdir("framers")
    video = None
    while True:
        start = time.perf_counter()
        ret, frame = cap.read()
        if not ret:
            print("无法读取视频帧")
            break
        # 是否保存帧
        if True:
            # if frame_number % 4 == 0:
            # 处理每一帧的操作，这里只是保存为图像文件

            result = model_instance.predict(frame)
            print(fd.vision.fd_result_to_json(result=result))
            print(f"第{frame_number}帧")
            vis_im = fd.vision.vis_detection(frame, result, score_threshold=0.5)

            if video is None:
                video = cv2.VideoWriter(
                    "__output__.mp4",
                    fourcc,
                    fps,
                    (vis_im.shape[1], vis_im.shape[0]),
                )
            video.write(vis_im)
        else:
            video.write(frame)
        end = time.perf_counter()
        cost = end - start
        print(f"spend time: {cost}\n\n")
        frame_number += 1


if __name__ == "__main__":
    run_app()
